// A short sketch that acts a bit like a spell checker.
// First read in a word list and build a dictionary from it.
// Then read through a text file and check each word to see
// whether it's in the word list.  Strip off punctation.
// Consolidate illegal words so that each one appears just
// once.

IntDict dict;

boolean isWord( String wd )
{
  return dict.hasKey( wd );
}

String trimPunctuation( String wd )
{
  int start = 0;
  while( !Character.isLetter( wd.charAt( start ) ) ) {
    ++start;
  }
  
  int end = wd.length() - 1;
  while( !Character.isLetter( wd.charAt( end ) ) ) {
    --end;
  }
  
  return wd.substring( start, end + 1 );
}

void setup()
{
  String[] wds = loadStrings( "words.txt" );
  dict = new IntDict();
  for( int idx = 0; idx < wds.length; ++idx ) {
    dict.set( wds[idx], 1 );
  }
  
  String[] txt = loadStrings( "input.txt" );
  String[] errors = new String[0];
  
  for( int idx = 0; idx < txt.length; ++idx ) {
    String[] lwds = splitTokens( txt[idx] );
    for( int widx = 0; widx < lwds.length; ++widx ) {
      String wd = lwds[widx].toLowerCase();
      wd = trimPunctuation( wd );
      if( !isWord( wd ) ) {
        errors = append( errors, wd );
      }
    }
  }
  errors = sort( errors );
  for( int idx = 0; idx < errors.length; ++idx ) {
    if( (idx==0) || (!errors[idx].equals(errors[idx-1])) ) {
      println( errors[idx] );
    }
  }
}
